@@ -200,7 +200,8 @@ void split(
 
 bool GenICamDriver::declareGenICamParameter(
   const std::string & ros_name,
-  const std::shared_ptr<GenApi::CNodeMapRef> & nodemap, const std::string & name)
+  const std::shared_ptr<GenApi::CNodeMapRef> & nodemap, const std::string & name,
+  const char *description, double float_scale)
 {
   bool ret = false;
 
@@ -212,7 +213,15 @@ bool GenICamDriver::declareGenICamParameter(
     if (node != 0) {
       if (GenApi::IsReadable(node) && GenApi::IsWritable(node)) {
         rcl_interfaces::msg::ParameterDescriptor param_descr;
-        param_descr.description = node->GetDescription();
+
+        if (description)
+        {
+          param_descr.description = description;
+        }
+        else
+        {
+          param_descr.description = node->GetDescription();
+        }
 
         switch (node->GetPrincipalInterfaceType()) {
           case GenApi::intfIBoolean:
@@ -253,18 +262,19 @@ bool GenICamDriver::declareGenICamParameter(
 
               rcl_interfaces::msg::FloatingPointRange float_range;
 
-              float_range.from_value = p->GetMin();
-              float_range.to_value = p->GetMax();
+              float_range.from_value = std::round(1e6*p->GetMin()/float_scale)/1e6;
+              float_range.to_value = std::round(1e6*p->GetMax()/float_scale)/1e6;
 
               float_range.step = 0;
               if (p->GetIncMode() == GenApi::fixedIncrement) {
-                float_range.step = p->GetInc();
+                float_range.step = p->GetInc()/float_scale;
               }
 
               param_descr.floating_point_range.push_back(float_range);
 
               param[ros_name] = name;
-              declare_parameter(ros_name, p->GetValue(false, false), param_descr);
+              param_float_scale[ros_name] = float_scale;
+              declare_parameter(ros_name, p->GetValue(false, false)/float_scale, param_descr);
               ret = true;
             }
             break;
@@ -331,7 +341,8 @@ bool GenICamDriver::declareGenICamParameter(
 bool GenICamDriver::declareGenICamParameter(
   const std::string & ros_name,
   const std::shared_ptr<GenApi::CNodeMapRef> & nodemap, const std::string & name,
-  const std::string & selector_name, const std::string & selector_value)
+  const std::string & selector_name, const std::string & selector_value,
+  const char *description, double float_scale)
 {
   std::lock_guard<std::recursive_mutex> lock(param_mtx);
   bool ret = false;
@@ -347,7 +358,7 @@ bool GenICamDriver::declareGenICamParameter(
 
     // declare parameter
 
-    ret = declareGenICamParameter(ros_name, nodemap, name);
+    ret = declareGenICamParameter(ros_name, nodemap, name, description, float_scale);
   } else {
     RCLCPP_WARN_STREAM(this->get_logger(), "Selector of parameter cannot be found or changed: " <<
       ros_name << " (" << selector_name << "=" << selector_value << ")");
@@ -476,7 +487,7 @@ void GenICamDriver::configure()
     }
   }
 
-  bool iocontrol_avail = nodemap->_GetNode("LineSource")->GetAccessMode() == GenApi::RW;
+  iocontrol_avail = nodemap->_GetNode("LineSource")->GetAccessMode() == GenApi::RW;
 
   // initialise variables for caching some values
 
@@ -484,6 +495,35 @@ void GenICamDriver::configure()
   update_exp_values = false;
   update_wb_values = false;
 
+  // Represent GenICam parameter ExposureAuto with ROS parameters
+  // camera_exp_control and camera_exp_auto_mode as in RestAPI interface
+  // of Roboception devices
+
+  {
+    std::lock_guard<std::recursive_mutex> lock(param_mtx);
+
+    std::string exp_auto=rcg::getString(nodemap, "ExposureAuto", false);
+
+    rcl_interfaces::msg::ParameterDescriptor param_descr;
+    param_descr.description = "Exposure control mode: [Manual, Auto, HDR]";
+    param_descr.additional_constraints = "Manual|Auto|HDR";
+
+    std::string val="Auto";
+    if (exp_auto == "Off") val="Manual";
+    if (exp_auto == "HDR") val="HDR";
+
+    declare_parameter("camera_exp_control", val, param_descr);
+
+    param_descr.description = "Auto-exposure mode: [Normal, Out1High, AdaptiveOut1]";
+    param_descr.additional_constraints = "Normal|Out1High|AdaptiveOut1";
+
+    val="Normal";
+    if (exp_auto == "Out1High") val="Out1High";
+    if (exp_auto == "AdaptiveOut1") val="AdaptiveOut1";
+
+    declare_parameter("camera_exp_auto_mode", val, param_descr);
+  }
+
   // register parameter callback
 
   param_cb = add_on_set_parameters_callback(std::bind(&GenICamDriver::paramCallback, this, _1));
@@ -496,16 +536,16 @@ void GenICamDriver::configure()
   // it.
 
   declareGenICamParameter("camera_fps", nodemap, "AcquisitionFrameRate");
-  declareGenICamParameter("camera_exp_auto", nodemap, "ExposureAuto");
-  declareGenICamParameter("camera_exp_max", nodemap, "ExposureTimeAutoMax");
+  declareGenICamParameter("camera_exp_max", nodemap, "ExposureTimeAutoMax", "Maximum exposure time in seconds", 1000000);
   declareGenICamParameter("camera_exp_auto_average_max", nodemap, "RcExposureAutoAverageMax");
   declareGenICamParameter("camera_exp_auto_average_min", nodemap, "RcExposureAutoAverageMin");
   declareGenICamParameter("camera_exp_width", nodemap, "ExposureRegionWidth");
   declareGenICamParameter("camera_exp_height", nodemap, "ExposureRegionHeight");
   declareGenICamParameter("camera_exp_offset_x", nodemap, "ExposureRegionOffsetX");
   declareGenICamParameter("camera_exp_offset_y", nodemap, "ExposureRegionOffsetY");
-  declareGenICamParameter("camera_exp_value", nodemap, "ExposureTime");
+  declareGenICamParameter("camera_exp_value", nodemap, "ExposureTime", "Exposure time in seconds", 1000000);
   declareGenICamParameter("camera_gain_value", nodemap, "Gain", "GainSelector", "All");
+  declareGenICamParameter("camera_gamma", nodemap, "Gamma");
 
   if (color) {
     declareGenICamParameter("camera_wb_auto", nodemap, "BalanceWhiteAuto");
@@ -734,26 +774,76 @@ rcl_interfaces::msg::SetParametersResult GenICamDriver::paramCallback(
   ret.successful = true;
 
   for (size_t i = 0; i < p.size(); i++) {
-    // signal processing thread to update manual exposure or white balancing
-    // values if automatic has been turned off
+    // signal processing thread to update exposure and gain values if automatic
+    // has been turned off
 
-    if (p[i].get_name() == "camera_exp_auto" && p[i].get_type() == rclcpp::PARAMETER_STRING &&
-      p[i].as_string() == "Off")
-    {
+    if (p[i].get_name() == "camera_exp_control" &&
+        (p[i].as_string() == "Manual" || p[i].as_string() == "HDR")) {
       update_exp_values = true;
     }
 
+    // translate camera_exp_control and camera_exp_auto_mode parameters into
+    // GenICam parameter ExposureAuto
+
+    if (p[i].get_name() == "camera_exp_control" || p[i].get_name() == "camera_exp_auto_mode") {
+      std::string exp_control, exp_auto_mode;
+
+      if (p[i].get_name() == "camera_exp_control") {
+        exp_control=p[i].as_string();
+      } else {
+        exp_control=get_parameter(std::string("camera_exp_control")).as_string();
+      }
+
+      if (p[i].get_name() == "camera_exp_auto_mode") {
+        exp_auto_mode=p[i].as_string();
+      } else {
+        exp_auto_mode=get_parameter(std::string("camera_exp_auto_mode")).as_string();
+      }
+
+      std::string val="Auto";
+      if (exp_control == "Manual") {
+        val="Off";
+      } else if (exp_control == "HDR") {
+        val="HDR";
+      } else if (exp_control == "Auto") {
+        if (exp_auto_mode == "Normal") {
+          val="Continuous";
+        } else if (exp_auto_mode == "Out1High" || exp_auto_mode == "AdaptiveOut1") {
+          val=exp_auto_mode;
+        } else {
+          RCLCPP_WARN_STREAM(this->get_logger(), "Ignoring illegal value '" << exp_auto_mode <<
+            "' of parameter camera_exp_auto_mode");
+          continue;
+        }
+      } else {
+        RCLCPP_WARN_STREAM(this->get_logger(), "Ignoring illegal value '" << exp_control <<
+          "' of parameter camera_exp_control");
+        continue;
+      }
+
+      try
+      {
+        rcg::setEnum(nodemap, "ExposureAuto", val.c_str(), true);
+      } catch (const std::exception & ex) {
+        RCLCPP_WARN_STREAM(this->get_logger(), "Cannot set parameter ExposureAuto to '" << val <<
+          "'. Check if device supports this value: " << ex.what());
+      }
+
+      continue;
+    }
+
+    // signal processing thread to update white balancing values if automatic
+    // has been turned off
+
     if (p[i].get_name() == "camera_wb_auto" && p[i].get_type() == rclcpp::PARAMETER_STRING &&
-      p[i].as_string() == "Off")
-    {
+      p[i].as_string() == "Off") {
       update_wb_values = true;
     }
 
     // skip if value is cached one
 
     if (p[i].get_name() == "out1_mode" && p[i].get_type() == rclcpp::PARAMETER_STRING &&
-      p[i].as_string() == remote_out1_mode)
-    {
+      p[i].as_string() == remote_out1_mode) {
       continue;
     }
 
@@ -788,7 +878,10 @@ rcl_interfaces::msg::SetParametersResult GenICamDriver::paramCallback(
           break;
 
         case rclcpp::PARAMETER_DOUBLE:
-          rcg::setFloat(nodemap, name.c_str(), p[i].as_double(), true);
+          {
+            double scale = param_float_scale.at(p[i].get_name());
+            rcg::setFloat(nodemap, name.c_str(), scale*p[i].as_double(), true);
+          }
           break;
 
         case rclcpp::PARAMETER_STRING:
@@ -1025,6 +1118,7 @@ void GenICamDriver::process()
               update_exp_values = false;
 
               double exp = rcg::getFloat(nodemap, "ExposureTime", 0, 0, true, true);
+              exp/=param_float_scale["camera_exp_value"];
               set_parameter(rclcpp::Parameter("camera_exp_value", exp));
 
               rcg::setEnum(nodemap, "GainSelector", "All", false);
@@ -1055,7 +1149,7 @@ void GenICamDriver::process()
               out1_mode_on_sensor = remote_out1_mode;
             }
 
-            if (out1_mode_on_sensor != remote_out1_mode) {
+            if (iocontrol_avail && out1_mode_on_sensor != remote_out1_mode) {
               remote_out1_mode = out1_mode_on_sensor;
               set_parameter(rclcpp::Parameter("out1_mode", out1_mode_on_sensor));
             }
